from convoy import Convoy
from convoy.data import *
from convoy.client import Client

Config["api_key"] = "your-api-key"

cnv = Convoy(Config)

#Application
def test_application():
    NewApplication["name"] = "pythonsdktestapp"
    NewApplication["support_email"] = "testsdk@frain.dev"
    _, status = cnv.applications.create({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, NewApplication)
    assert(status == 201)
    # response = cnv.applications.all("groupID=449f4d86-e70b-40eb-a733-dadd89d8d3b6")
    # response = cnv.applications.find("fb05c3c7-2d9c-4e07-95b4-e859c1415385", "groupID=449f4d86-e70b-40eb-a733-dadd89d8d3b6")
    # UpdateApplication = NewApplication
    # UpdateApplication["name"] = "sdktest_app"
    # response = cnv.applications.update("9a3cc2b9-ce09-4dbd-9ea9-594def3dd347", "groupID=449f4d86-e70b-40eb-a733-dadd89d8d3b6", UpdateApplication)

#Endpoint
def test_endpoint():
    NewEndpoint["url"] = "http://ed0b-102-219-153-85.ngrok.io"
    NewEndpoint["description"] = "testendpoint"
    _, status = cnv.endpoint.create("9a3cc2b9-ce09-4dbd-9ea9-594def3dd347", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, NewEndpoint)
    assert(status == 201)
    # response = cnv.endpoint.all("fb05c3c7-2d9c-4e07-95b4-e859c1415385", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    # response = cnv.endpoint.find("fb05c3c7-2d9c-4e07-95b4-e859c1415385", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, "7ebfde76-ad24-4913-b516-72a8a8e14735")
    # UpdateEndpoint = NewEndpoint
    # UpdateEndpoint["description"] = "test_endpoint"
    # response = cnv.endpoint.update("a535e91a-2fd2-40e4-95d5-138df62a63db","5523df4f-0bf0-4d6a-9c36-064306b8b7d2", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, UpdateEndpoint)
    # response = cnv.endpoint.delete("a535e91a-2fd2-40e4-95d5-138df62a63db","5523df4f-0bf0-4d6a-9c36-064306b8b7d2", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, "")

#EventDelivery
def test_eventdelivery():    
    content, status = cnv.eventDelivery.all({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    assert(len(content) > 0)
    assert(status == 200)
    # response = cnv.eventDelivery.find("dfa306bd-ef49-4528-8175-a363e01c4623", {})

#Event
def test_event():
    NewEvent["app_id"] = "9a3cc2b9-ce09-4dbd-9ea9-594def3dd347"
    NewEvent["event_type"] = "payment.success"
    NewEvent["data"] = {
                        "event": "payment.success",
                        "data": {
                            "status": "Completed",
                            "description": "Transaction Successful",
                            "userID": "test_user_id808",
                            "paymentReference": "test_ref_85149",
                            "amount": 200,
                            "senderAccountName": "Alan Christian Segun",
                            "sourceAccountNumber": "2999993564",
                            "sourceAccountType": "personal",
                            "sourceBankCode": "50211",
                            "destinationAccountNumber": "00855584818",
                            "destinationBankCode": "063"
                        },
                    }
    content, status = cnv.event.create({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, NewEvent)
    assert(len(content) > 0)
    assert(status == 201)
    # response = cnv.event.all({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    # response = cnv.event.find("89b0574d-24fd-44a8-b902-bb679b543506", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})

# #Group
def test_group():
    NewGroup = {
                "name": "pythonsdk",
                "logo_url": "",
                "config": {
                        "disable_endpoint": False,
                        "signature": {
                                    "hash": "SHA256",
                                    "header": "X_PYTHONSDK_APP"
                                    },
                        "strategy": {
                                    "type": "default",
                                    "default": {
                                        "intervalSeconds": 5,
                                        "retryLimit": 3
                                }
                            },
                        },
                    }

    # content, status = cnv.group.create({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, NewGroup)
    content, status = cnv.group.all({"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    assert(status == 200)
    # response = cnv.group.find("96d04e60-854c-4f7e-9257-be88c15474fd", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    # UpdateGroup = NewGroup
    # UpdateGroup["name"] = "convoypythonsdk"
    # response  = cnv.group.update("f46f780a-4216-4306-b1ba-1c5e0a30086e", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"}, UpdateGroup)

#DeliveryAttempts
def test_deliverattempts():
    content, status = cnv.deliveryAttempts.all("6bb26fe3-a3d8-4ea8-aa7c-55bd61caa8be", {"groupID":"449f4d86-e70b-40eb-a733-dadd89d8d3b6"})
    assert(status == 404)
    # response  = cnv.deliveryAttempts.find("dfa306bd-ef49-4528-8175-a363e01c4623", "1d1e3b81-3a33-472b-9380-a8c8afa06252", {})



from convoy.client import Client

class Event():
    """Initializes an Event object to make calls to the /event endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, query):
        response = self.client.httpGet("/events", query)
        return response

    def create(self, query, data):
        '''
        Create a new event.
        Parameters
        ----------
        data = {
                app_id: "",
                event_type: "",
                data: {
                        "event": "",
                        "data": {},
                    }
                }
        '''
        response = self.client.httpPost("/events", query, data)
        return response

    def find(self, id, query):
        response = self.client.httpGet("/events/%s" % id, query)
        return response




from convoy.api.application import Application
from convoy.api.delivery_attempts import DeliveryAttempt
from convoy.api.endpoint import Endpoint
from convoy.api.event import Event
from convoy.api.group import Group


import requests

def responseHelper(e):
    if e == requests.exceptions.HTTPError:
        print ("Http Error:", e)
    if e == requests.exceptions.ConnectionError:
        print ("Error Connecting:", e)
    if e == requests.exceptions.Timeout:
        print ("Timeout Error:", e)
    if e == requests.exceptions.RequestException:
        print(e)


'''
You can use these dict objects to create your own data.
'''

Config = {
        #Username used for basic authentication
        "username": "",
        #Password used for basic authentication
        "password": "",
        #API Key used for bearer token authentication
        "api_key": "",
        #Convoy self hosted uri
        "uri": ""
    }

NewApplication = {
                "name": "",
                "support_email": "",
                "secret": ""
                }


NewEvent = {
            "app_id": "",
            "event_type": "",
            "data": {
                    "event": "",
                    "data": {},
                }
            }

NewEndpoint = {
                "url": "",
                "description": "",
                "secret": "",
                "events": [],
                }

BatchResend = {
                "ids": []
            }


NewGroup = {
            "name": "",
            "logo_url": "",
            "config": {
                    "disable_endpoint": bool,
                    "signature": {
                                "hash": "",
                                "header": ""
                                },
                    "strategy": {
                                "type":   "",
                                "default": {
                                    "intervalSeconds": int,
                                    "retryLimit": int
                            }
                        },
                    },
                }






from convoy.client import Client

class Group():
    """Initializes a Groups object to make calls to the /groups endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, query):
        '''
        Get all groups.
        '''
        response = self.client.httpGet("/groups", query)
        return response

    def create(self, query, data):
        '''
        Create a new group.
        Parameters
        ----------
        data = {
            "name": "",
            "logo_url": "",
            "config": {
                    "disable_endpoint": bool,
                    "signature": {
                                "hash": "",
                                "header": ""
                                },
                    "strategy": {
                                "type":   "",
                                "default": {
                                    "intervalSeconds": int,
                                    "retryLimit": int
                            }
                        },
                    },
                }
        '''
        response = self.client.httpPost("/groups", query, data)
        return response

    def find(self, id, query):
        '''
        Find a particular group.    
        '''
        response = self.client.httpGet("/groups/%s" % id, query)
        return response

    def update(self, id, query, data):
        '''
        Update a group.
        Parameters
        ---------- 
        data = {
            "name": "",
            "logo_url": "",
            "config": {
                    "disable_endpoint": bool,
                    "signature": {
                                "hash": "",
                                "header": ""
                                },
                    "strategy": {
                                "type":   "",
                                "default": {
                                    "intervalSeconds": int,
                                    "retryLimit": int
                            }
                        },
                    },
                }
        '''
        response = self.client.httpPut("/groups/%s" % id, query, data)
        return response

    def delete(self, id, query, data):
        '''
        Delete a group.
        '''
        response = self.client.httpDelete("/groups/%s" % id, query, data)
        return response




from convoy.api import application, delivery_attempts, endpoint, event, event_delivery, group

class Convoy():
    """Initializes the main Convoy Object.
       
    Parameters
    ----------
    config : dict of config values, see example config below;

    config = {
        #Username used for basic authentication
        "username": "",
        #Password used for basic authentication
        "password": "",
        #API Key used for bearer token authentication
        "api_key": "",
        #Convoy self hosted uri
        "uri": ""
    }
    """
    def __init__(self, config):
        self.applications = application.Application(config)
        self.deliveryAttempts = delivery_attempts.DeliveryAttempt(config)
        self.endpoint = endpoint.Endpoint(config)
        self.eventDelivery = event_delivery.EventDelivery(config)
        self.event = event.Event(config)
        self.group = group.Group(config)


from convoy.client import Client

class Endpoint():
    """Initializes an Endpoint object to make calls to the /endpoints endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, appId, query):
        '''
        Get all endpoints for an application.
        '''
        response = self.client.httpGet("/applications/%s/endpoints" % appId, query)
        return response

    def create(self, appId, query, data):
        '''
        Create a new endpoint.
        Parameters
        ----------
        data = {
                "url": "",
                "description": "",
                "secret": "",
                "events": [],
                }
        '''
        response = self.client.httpPost("/applications/%s/endpoints" % appId, query, data)
        return response

    def find(self, appId, endpointId, query):
        '''
        Find a particular application.
        '''
        response = self.client.httpGet("/applications/%s/endpoints/%s" % (appId, endpointId), query)
        return response

    def update(self, appId, endpointId, query, data):
        '''
        Update an application.
        Parameters
        ---------- 
        data = {
                "url": "",
                "description": "",
                "secret": "",
                "events": [],
                }
        '''
        response = self.client.httpPut("/applications/%s/endpoints/%s" % (appId, endpointId), query, data)
        return response

    def delete(self, appId, endpointId, query, data):
        '''
        Delete an application.
        '''
        response = self.client.httpDelete("/applications/%s/endpoints/%s" % (appId, endpointId), query, data)
        return response




from convoy.client import Client

class DeliveryAttempt():
    """Initializes a DeliveryAttempt object to make calls to the /deliveryattempts endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, eventdeliveryId, query):
        response = self.client.httpGet("/eventdeliveries/%s/deliveryattempts" %  eventdeliveryId, query)
        return response

    def find(self, eventdeliveryId, deliveryAttemptId, query):
        response = self.client.httpGet("/eventdeliveries/%s/deliveryattempts/%s" %  (eventdeliveryId, deliveryAttemptId), query)
        return response




from convoy.client import Client

class Application():
    """Initializes an Application object to make calls to the /application endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, query):
        '''
        Get all applicaitons.
        '''
        response = self.client.httpGet("/applications", query)
        return response

    def create(self, query, data):
        '''
        Create a new application.
        Parameters
        ----------
        data = {
                "name": "",
                "support_email": ""
                }
        '''
        response = self.client.httpPost("/applications", query, data)
        return response

    def find(self, id, query):
        '''
        Find a particular application.    
        '''
        response = self.client.httpGet("/applications/%s" % (id), query)
        return response

    def update(self, id, query, data):
        '''
        Update an application.
        Parameters
        ---------- 
        data = {
                "name": "",
                "support_email": ""
                }
        '''
        response = self.client.httpPut("/applications/%s" % id, query, data)
        return response

    def delete(self, id, query, data):
        '''
        Delete an application.
        '''
        response = self.client.httpDelete("/applications/%s" % id, query, data)
        return response




import requests
import json
from convoy.utils import responseHelper
from base64 import b64encode

class Client():
    """
    Initializes a Client Object.
    """
    def __init__(self, config):
        self.apiKey = config["api_key"]
        self.username = config["username"]
        self.password = config["password"]

        if config["uri"] == "":
            self.baseUri = 'https://cloud.getconvoy.io/api/v1'
        else:
            self.baseUri = config["uri"]

        self.headers = {'Authorization': self.getAuthorization(), 'Content-Type': 'application/json; charset=utf-8'}

    def httpGet(self, path, query):
        try:
            response = requests.get(self.buildPath(path), headers=self.headers, params=query)
            return response.json(), response.status_code
        except BaseException as e:
            return responseHelper(e) 

    def httpPost(self, path, query, data):
        try:
            response = requests.post(self.buildPath(path), data=json.dumps(data), headers=self.headers, params=query)
            return response.json(), response.status_code
        except BaseException as e:
            return responseHelper(e) 

    def httpPut(self, path, query, data):
        try:
            response = requests.put(self.buildPath(path), data=json.dumps(data), headers=self.headers, params=query)
            return response.json(), response.status_code
        except BaseException as e:
            return responseHelper(e) 

    def httpDelete(self, path, query, data):
        try:
            response = requests.delete(self.buildPath(path), data=json.dumps(data), headers=self.headers, params=query)
            return response.json(), response.status_code
        except BaseException as e:
            return responseHelper(e) 

    def getBaseUrl(self):
        return self.baseUri

    def getAuthorization(self):
        if self.apiKey != "":
            return "Bearer %s" % self.apiKey

        return "Basic %s" % b64encode(("%s:%s" % (self.username, self.password)).encode('utf-8')).decode('utf-8')

    def buildPath(self, path):
        return "%s%s" % (self.baseUri, path)



"""Setup script for realpython-reader"""

# Standard library imports
import pathlib

# Third party imports
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).resolve().parent

# The text of the README file is used as a description
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="convoy-python",
    version="0.1.0",
    description="Python SDK for Convoy",
    url="https://github.com/frain-dev/convoy-python",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Frain Inc.",
    author_email="info@frain.dev",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=["convoy", ".convoy/client", ".convoy/api", ".convoy/utils"],
    include_package_data=True,
    install_requires=["requests"],
)


from convoy.client import Client

class EventDelivery():
    """Initializes an EventDelivery object to make calls to the /eventdeliveries endpoint.
       
    Parameters
    ----------
    config : dict of config values
    """
    def __init__(self, config):
        self.client = Client(config)

    def all(self, query):
        '''
        Get all eventdeliveries.
        '''
        response = self.client.httpGet("/eventdeliveries", query)
        return response

    def find(self, id, query):
        '''
        Find a particular eventdelivery.    
        '''
        response = self.client.httpGet("/eventdeliveries/%s" % id, query)
        return response

    def resend(self, id, query):
        '''
        Resend an eventdelivery.    
        '''
        response = self.client.httpPut("/eventdeliveries/%s/resend" % id, query, {})
        return response

    def batchresend(self, id, query, data):
        '''
        Batch resend eventdeliveries.
        Parameters
        ----------
        data = {
                ids: []
                }
        '''
        response = self.client.httpPut("/eventdeliveries/batchretry" % id, query, data)
        return response

